/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "board-gui.h"
using namespace Pasang;

BoardGui::BoardGui ()
{
  set_turn (PLAYER_VIRTUAL, PLAYER_HUMAN, 1);
  kas_point.x = AT_REST;
  set_size_request (theme.board_image_width (), theme.board_image_width ());
  add_events (Gdk::POINTER_MOTION_MASK);
  add_events (Gdk::BUTTON_MOTION_MASK);
  add_events (Gdk::BUTTON_PRESS_MASK);
  add_events (Gdk::BUTTON_RELEASE_MASK);
  for (int position=0; position < MATRIX_SIZE; position++) {
    frame[position] = 0;
    frame_stop[position] = 0;
    frame_count[position] = 1;
  }
  move_submission_stage = 0;
  prev_time = 0;
}

BoardGui::~BoardGui ()
{
}

/**
 * top_player: Player who "sits" at the top
 * bottom_player: Player who "sits" at the bottom
 * first_player: who plays first, top_player (0) or bottom_player (1)
 */
void 
BoardGui::set_turn (PlayerType top_player, PlayerType bottom_player, int first_player)
{
  g_assert (first_player == 0 || first_player == 1);
  rotated = first_player;
  player_type[0 ^ rotated] = top_player;
  player_type[1 ^ rotated] = bottom_player;
}

/**
 * Who is playing next turn: PLAYER_HUMAN, or PLAYER_VIRTUAL, or ...?
 */
PlayerType 
BoardGui::get_player_type ()
const
{
  return player_type[ board.get_player () ];
}

/**
 * Who is playing next turn: the player sitting at the top or the one at the bottom?
 */
PlayerSideType 
BoardGui::get_player_side ()
const
{
  return (PlayerSideType)(board.get_player () ^ rotated);
}

/**
 * Score for the player at the top/bottom.
 * Not to be confused with Board::get_score(int playerNumber)
 */
int 
BoardGui::get_score (PlayerSideType ps) 
const
{
  return board.get_score ((int)(ps ^ rotated));
}

bool 
BoardGui::on_expose_event (GdkEventExpose* event)
{
  const GameStageType stage = board.get_stage ();
  const int player = board.get_player ();
  Glib::RefPtr<Gdk::Window> win = get_window ();           
  Glib::RefPtr<Gdk::GC> gc = get_style()->get_black_gc ();
  theme.draw_board (win, gc, 0, 0);

  for (int pass=0; pass < 30; pass++) {
    for (int position=0; position < MATRIX_SIZE; position++) {
      if (frame[position] != pass) continue;

      // Skip blanks and guards
      PieceType piece = board.get_piece (position);
      if (piece > PIECE_COUNT) continue;
      
      // If we are making a kas, the source square should be shown empty
      // (actually a piece is still there in the matrix because the move
      // is not completed/submitted yet).
      if (stage == STAGE_SELECT_KAS && 
          selected_move != NULL && 
          position == selected_move->position) {
        continue;
      }
      // If we are dragging a kas, the source square should be empty.
      // (actually the kas is still there in the matrix, but we are going to
      // show the kas being dragged after this loop).
      if (stage == STAGE_MOVE_KAS && 
          kas_point.x != AT_REST && 
          piece == board.get_kas ()) {
        continue;
      }                     
      const Point p = to_point (position);
      const int radius = theme.piece_image_width (piece) / 2;
      theme.draw_piece (win, gc, piece, frame[position],
                        p.x - radius,  p.y - radius);
    }//endfor position
    // Draw dragged kas
    if (frame[0] != pass) continue;
    if ((stage == STAGE_SELECT_KAS && selected_move != NULL) || 
        (stage == STAGE_MOVE_KAS && kas_point.x != AT_REST)) {
      const PieceType piece = board.get_kas ();
      const int radius = theme.piece_image_width (piece) / 2;
      theme.draw_piece (win, gc, piece, frame[0],
                        kas_point.x - radius,  kas_point.y - radius);
    }
  }//endfor pass
  
  return DrawingArea::on_expose_event (event);
}

void 
BoardGui::queue_draw_piece (const int position)
{
  Point p = to_point (position);
  PieceType piece = board.get_piece (position);
  int width = piece == PIECE_EMPTY? theme.cell_width () : theme.piece_image_width (piece);
  queue_draw_area (p.x - width / 2, p.y - width / 2,  width, width);
}

void
BoardGui::queue_draw_kas () {
  int width = theme.piece_image_width (board.get_kas ());
  queue_draw_area (kas_point.x - width / 2, kas_point.y - width / 2, width, width);
}

/**
 * Mouse motion handler is divided into 2 parts. on_motion_notify_event()
 * gets the actual mouse movement, ignoring it if it is not the human's turn.
 * Actual process is done by mouse_moved(), which is also the entry point
 * for simulated mouse movement.
 */
bool
BoardGui::input_allowed ()
{
  return get_player_type () == PLAYER_HUMAN
        && move_submission_stage == 0;
}
 
bool 
BoardGui::on_motion_notify_event (GdkEventMotion* event)
{
  if (input_allowed ())
    mouse_moved (Point ((int) (event->x), (int) (event->y)));
  return true;
}

void 
BoardGui::mouse_moved (Point mouse)
{
  switch (board.get_stage ()) {
    case STAGE_OPENING :
          animate_drop ();
          animate_pick (find_move (mouse));
          break;
    case STAGE_RESOLVE_KAS_SELECTION :
    case STAGE_RESOLVE : 
          find_move (mouse);
          break;
    case STAGE_SELECT_KAS : 
          drag_candidate (mouse);
          break;
    case STAGE_MOVE_KAS :
          drag_kas (mouse);
          break;
  }
}

/**
 * Mouse press handler is divided into 2 parts. on_button_press_event()
 * gets the actual mouse press, ignoring it if it is not the human's turn.
 * Actual process is done by mouse_pressed(), which is also the entry point
 * for simulated mouse.
 */
bool 
BoardGui::on_button_press_event (GdkEventButton* event)
{
  if (input_allowed () && (event->button == 1))
    mouse_pressed (Point ((int) (event->x), (int) (event->y)));
  return true;
}

void 
BoardGui::mouse_pressed (Point mouse)
{
  switch (board.get_stage ()) {
    case STAGE_OPENING :
    case STAGE_RESOLVE_KAS_SELECTION :
    case STAGE_RESOLVE : 
          if (find_move (mouse)) {
            board.singularize_branch (*selected_move);
            submit_selected_move ();
          }
          break;
    case STAGE_SELECT_KAS : 
          pick_candidate (mouse);
          break;
    case STAGE_MOVE_KAS :
          pick_kas (mouse);
          break;
  }
}
 
/**
 * Mouse release handler is divided into 2 parts. on_button_release_event()
 * gets the actual mouse release, ignoring it if it is not the human's turn.
 * Actual process is done by mouse_released(), which is also the entry point
 * for simulated mouse.
 */
bool 
BoardGui::on_button_release_event (GdkEventButton* event)
{
  if (input_allowed () && (event->button == 1))
    mouse_released (Point ((int) (event->x), (int) (event->y)));
  return true;
}

void 
BoardGui::mouse_released (Point mouse)
{
  switch (board.get_stage ()) {
    case STAGE_SELECT_KAS :
            drop_candidate (mouse);
            break;
    case STAGE_MOVE_KAS :
            drop_kas (mouse);
            break;
  }
}

/**
 * This is called when PatternSelector::on_selection is notified of a change
 */
void 
BoardGui::on_pattern_changed (const Matrix m)
{
  board.start_game (m);
  simulating_mouse = false;
  move_submission_stage = 0;
  animate_drop ();
  animate_drop_kas ();
  queue_draw ();
} 

/**
 * Perform the move. To be called only when the mouse gesture has been completed.
 */
void 
BoardGui::submit_selected_move ()
{
  g_assert (selected_move != NULL);
  move_submission_stage = 1;
  if (board.count_ambiguous_forks (*selected_move) == 1) {
    animate_remove (selected_move);
  }
  // Snap kas
  kas_point = to_point (to_position (kas_point));
}

void
BoardGui::perform_selected_move ()
{
  g_assert (move_submission_stage == 2);
  g_assert (selected_move != NULL);
  std::string move_notation = selected_move->to_string ();
  board.perform (*selected_move, get_player_type () != PLAYER_HUMAN);
  int stage = board.get_stage ();
  if (stage == STAGE_RESOLVE_KAS_SELECTION || stage == STAGE_RESOLVE) 
    move_notation = std::string ("--");
  animate_drop ();
  queue_draw ();
  selected_move = NULL;
  kas_point.x = AT_REST;
  move_submission_stage = 0;
  next_move_signal.emit (&move_notation);
}

/**
 * Find the move that captures the piece at the presumed mouse location.
 * Side effect: set selected_move
 */
Move* 
BoardGui::find_move (const Point mouse)
{
  selected_move = NULL;
  const int position = to_position (mouse);
  for (int n=0; n < board.count_branches (); n++) {
    Move& move = board.get_branch (n);
    for (int b=0; b < move.count_booties (); b++) {
      if (position == move.get_booty_position (b)) {
        selected_move = &move;
        return selected_move;
      }
    }
  }
  return NULL;
}

/**
 * Select a piece to be promoted to kas. The candidate must be within
 * the vicinity of the mouse pointer.
 * Side effect: set selected_move, kas_point, drag_point
 */
void 
BoardGui::pick_candidate (const Point mouse)
{
  selected_move = NULL;
  const int position = to_position (mouse);
  for (int n=0; n < board.count_branches (); n++) {
    Move& move = board.get_branch (n);
    if (position == move.position) {
      selected_move = &move;
      kas_point = to_point (position);
      drag_point.x = kas_point.x - mouse.x;
      drag_point.y = kas_point.y - mouse.y;
      queue_draw_kas ();
      animate_pick_kas ();
      break;
    }
  }
}

/**
 * If there was a selected candidate, drag it. If it goes to the
 * empty column (which was captured during the opening move), highlight 
 * all its possible captures.
 * Side effect: set candidate_in
 */
void 
BoardGui::drag_candidate (const Point mouse)
{
  if (selected_move == NULL) return;
  // Update the kas position
  queue_draw_kas ();
  kas_point.x = mouse.x + drag_point.x;
  kas_point.y = mouse.y + drag_point.y;
  queue_draw_kas ();
        
  // See if the kas is dragged to its proper place
  int pos = to_position (kas_point);
  int row = pos / MATRIX_WIDTH;
  int col = pos % MATRIX_WIDTH;
  int kas_row      = selected_move->position / MATRIX_WIDTH;
  int empty_column = board.get_first_move (board.get_player ());
  candidate_in = (row == kas_row  &&  col == empty_column);
  if (candidate_in) {
    // Show all possible captures
    for (int i=board.count_ambiguous_forks (*selected_move) - 1;  i >= 0;  i--) {
      animate_pick (&(board.get_branch (selected_move->id + i)));
    }
    // In case the kas is dropped, invalidate its drop square.
    queue_draw_piece (pos);
  }
  else {
    animate_drop ();
  }
  animate_pick_kas ();
}
 
/**
 * Promote the selected candidate to kas, if it is already dragged home.
 */
void 
BoardGui::drop_candidate (const Point mouse)
{
  if (selected_move == NULL) return;
  drag_candidate (mouse);    // Just to set candidate_in
  if (candidate_in) {
    submit_selected_move ();
  }
  else {
    // Invalidate candidate's square
    queue_draw_piece (selected_move->position);
    selected_move = NULL;
  }
  animate_drop_kas ();
}
    
/**
 * Pick the kas if the mouse is pointing at it.
 * Effect: set kas_point, drag_point
 */
void 
BoardGui::pick_kas (Point mouse)
{
  int pos = to_position (mouse);
  if (pos == board.get_kas_position (board.get_player ())) {
    kas_point = to_point (pos);
    queue_draw_kas ();
    drag_point.x = kas_point.x - mouse.x;
    drag_point.y = kas_point.y - mouse.y;
    animate_pick_kas ();
  }
  else {
    kas_point.x = AT_REST;
  }
}
    
/**
 * Drag the kas to follow the mouse. The kas must be previously picked.
 */
void 
BoardGui::drag_kas (Point mouse)
{
  if (kas_point.x == AT_REST) return;
  // Update kas position
  queue_draw_kas ();
  kas_point.x = mouse.x + drag_point.x;
  kas_point.y = mouse.y + drag_point.y;
  queue_draw_kas ();

  int pos = to_position (kas_point);
  selected_move = NULL;
  animate_drop ();
  // See if the kas position is listed in the move list
  for (int i=0; i < board.count_branches (); i++) {
    Move& move = board.get_branch (i);
    if (move.position == pos) {
      selected_move = &move;
      // Show all the possible captures
      for (int n=board.count_ambiguous_forks (move) - 1;  n >= 0;  n--) {
        animate_pick (&(board.get_branch (move.id + n)));
      }
      break;
    }
  }
  animate_pick_kas ();
}
    
/**
 * Drop the kas at the mouse location.
 */
void 
BoardGui::drop_kas (Point mouse)
{
  if (kas_point.x == AT_REST) return;
  drag_kas (mouse);   // Just to set selected_move
  if (selected_move != NULL) {
    submit_selected_move ();
  }
  else {
    kas_point.x = AT_REST;
    queue_draw_piece (board.get_kas_position (board.get_player ()));
  }
  animate_drop_kas ();
}    

/**
 * Visual feedback of the pieces to be picked and removed (or picked and moved for kas)
 */
void
BoardGui::animate_pick (const int pos, const PieceType p) {
  PieceType piece = p == PIECE_NULL? board.get_piece (pos) : p;
  if (piece == PIECE_EMPTY) piece = board.get_kas ();
  // In the formula below, "+1" caters for odd number of frames.
  int num_frames = theme.count_frames (piece);
  frame_stop[pos] = (num_frames + 1) / 2 - 1;
  frame_count[pos] = num_frames;
}

void 
BoardGui::animate_pick (const Move* move)
{
  if (move == NULL) return;
  const int num_booties = move->count_booties ();
  for (int i=0; i < num_booties; i++) {
    animate_pick (move->get_booty_position (i));
  }
}

void
BoardGui::animate_pick_kas ()
{
  animate_remove (0, board.get_kas ());
}

void 
BoardGui::animate_remove (const int pos, const PieceType p)
{
  PieceType piece = p == PIECE_NULL? board.get_piece (pos) : p;
  int num_frames = theme.count_frames (piece);
  frame_stop[pos] = num_frames - 1;
  frame_count[pos] = num_frames;
}

void 
BoardGui::animate_remove (const Move* move)
{
  if (move == NULL) return;
  const int num_booties = move->count_booties ();
  for (int i=0; i < num_booties; i++) {
    animate_remove (move->get_booty_position (i));
  }
}

void 
BoardGui::animate_drop ()
{
  for (int pos=1; pos < MATRIX_SIZE; pos++) {
    frame_stop[pos] = 0;
  }
}

void
BoardGui::animate_drop_kas () 
{
  frame_stop[0] = 0;
}


/**
 * Conversion between display coordinates and matrix position
 */
Point 
BoardGui::to_point (int pos)
{
  Point p;
  p.x = pos % MATRIX_WIDTH;
  p.y = pos / MATRIX_WIDTH;
  if (rotated) {
    p.x = MATRIX_WIDTH - 1 - p.x;
    p.y = MATRIX_WIDTH - 1 - p.y;
  }
  p.x *= theme.cell_width ();
  p.y *= theme.cell_width ();
  // Skip border
  p.x += theme.cell_width () / 2;
  p.y += theme.cell_width () / 2;
  return p;
}
    
int 
BoardGui::to_position (const Point &p)
{
  // Locate the nearest piece
  int col = p.x / theme.cell_width ();
  int row = p.y / theme.cell_width ();
  if (col < 1 || col > 11 || row < 1 || row > 11) return -1;
  if (rotated) {
    col = MATRIX_WIDTH - 1 - col;
    row = MATRIX_WIDTH - 1 - row;
  }
  int pos = row * MATRIX_WIDTH + col;
  // Get the centre of this piece
  Point c = to_point (pos);
  // Relative position of p to c
  int x = p.x - c.x;
  int y = p.y - c.y;
  PieceType piece = board.get_piece (pos);
  
  int snap_range = 1 + (simulating_mouse ? 1 : theme.cell_width () / 6);  
  if // p is near c. Snap, even if the piece has a hole in the centre.
     ((p.x - c.x) * (p.x - c.x) + (p.y - c.y)*(p.y - c.y) <= snap_range * snap_range
     // p is near a piece. Snap to the piece.
     || !simulating_mouse 
        && piece < PIECE_COUNT 
        && theme.detect_piece (piece, 0, x, y)
     // p is near a vacant position. Snap a dragged kas to that position.
     || !simulating_mouse
        && piece == PIECE_EMPTY 
        && (theme.detect_piece (board.get_kas (), 0, -x, -y)
            || 
            theme.detect_piece (board.get_kas (), frame[0], -x, -y))) {
    return pos;
  }
  else {
    return -1;
  }
}

/**
 * Simulate mouse movement to effect the given move.
 */
void 
BoardGui::simulate (Move& move)
{
  g_assert (! simulating_mouse);
  board.singularize_branch (move);
  const int player = board.get_player ();
  int row = 0;
  int col = 0;
  switch (board.get_stage ()) {
    case STAGE_OPENING : 
            row = (player == 0) ? 1 : 7;
            simulated_press_pos = move.position + row * MATRIX_WIDTH;
            simulated_release_pos = simulated_press_pos;
            break;
    case STAGE_SELECT_KAS :
            row = move.position / MATRIX_WIDTH;
            col = board.get_first_move (player);
            simulated_press_pos = move.position;
            simulated_release_pos = col + row * MATRIX_WIDTH;
            break;
    case STAGE_MOVE_KAS :
            simulated_press_pos = board.get_kas_position (player);
            simulated_release_pos = move.position;
            break;
  }
  simulating_mouse = true;
  start_of_simulation = true;
}

void 
BoardGui::stop_simulation ()
{
  move_submission_stage = 0;
  simulating_mouse = false;
  kas_point.x = AT_REST;
  selected_move = NULL;
  animate_drop ();
  queue_draw ();
}


void
BoardGui::animate ()
{
  double time = timer.elapsed () / 1.0;  // Time span to play the animation: 1.0 sec
  if (simulating_mouse) {
    if (start_of_simulation) {
      timer.reset ();
      time = 0;
    }
    animate_simulated_mouse_events (time);
  }
  
  if (move_submission_stage == 1) {
    // Remove pieces to be captured
    move_submission_stage = 2;
    timer.reset ();
  }
  else if (move_submission_stage == 2) {
    double stop_time = (get_player_type () == PLAYER_HUMAN) ? 0.5 : 1.0;
    if (time >= stop_time) perform_selected_move ();
  }
  else if (get_player_type () == PLAYER_HUMAN) {
    // Show moves to be resolved
    int stage = board.get_stage ();
    if (stage == STAGE_RESOLVE_KAS_SELECTION || stage == STAGE_RESOLVE) {
      int num_moves = board.count_branches ();
      g_assert (num_moves > 1);
      int n = (int)(time / 0.75) % num_moves; // Choose a move to highlight
      animate_drop ();
      animate_pick (selected_move != NULL ? selected_move : &(board.get_branch (n)));
    }
  }
  
  // Advance animation frames
  for (int pos=0; pos < MATRIX_SIZE; pos++) {
    if ((frame[pos] == frame_stop[pos]) 
        || ((int)(time * frame_count[pos]) == (int)(prev_time * frame_count[pos])))
      continue;
    if (frame[pos] < frame_stop[pos])
      frame[pos]++;
    else
      frame[pos]--;
    if (pos != 0) {
      queue_draw_piece (pos);
    }
    else if (kas_point.x != AT_REST) {
      queue_draw_kas ();
    }
  }
  prev_time = time;
}

void
BoardGui::animate_simulated_mouse_events (double time)
{
  g_assert (get_player_type () != PLAYER_HUMAN);
  Point simulated_press_point = to_point (simulated_press_pos);
  Point simulated_release_point = to_point (simulated_release_pos);
  if (board.get_stage() == STAGE_OPENING) {
    if (start_of_simulation) {
      mouse_moved (simulated_press_point);
    }
    else if (time > 1) {
      mouse_pressed (simulated_press_point);
      simulating_mouse = false;
    }
  }
  else {
    if (start_of_simulation) {
      mouse_pressed (simulated_press_point);  // Press and "move" immediately because
      mouse_moved (simulated_press_point);    // processor slice may end anytime
    }
    else  {
      // How far the piece has moved from press point (distance 0) 
      // to release point (distance 1).
      // Don't let the piece move beyond release point (distance 1)
      const double time_to_pick_kas = 0.5;
      const double distance = MIN (MAX (0, time - time_to_pick_kas), 1.0);
      const int x = (int) (simulated_press_point.x * (1 - distance) + 
                           simulated_release_point.x * distance);
      const int y = (int) (simulated_press_point.y * (1 - distance) + 
                           simulated_release_point.y * distance);
      mouse_moved (Point (x, y));
      if (distance == 1.0) {
        mouse_released (simulated_release_point);
        simulating_mouse = false;
      }
    }
  }
  start_of_simulation = false;
}

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
