/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "board.h"
using namespace Pasang;


Board::Board ()
{
  player   = 0;
  num_branches  = 0;
  stage = STAGE_NULL;
}    
    
/**
 * Clone a board, except for its list of moves.
 */
Board::Board (const Board &b)
{
  for (int i=0; i<MATRIX_SIZE; i++) {
    matrix[i] = b.matrix[i];
  }
  player = b.player;
  value_of_uncaptured_pieces = b.value_of_uncaptured_pieces;
  stage = b.stage;
  score[0] = b.score[0];
  score[1] = b.score[1];
  kas_position[0] = b.kas_position[0];
  kas_position[1] = b.kas_position[1];
  first_move[0] = b.first_move[0];
  first_move[1] = b.first_move[1];
  // Don't copy moves
  num_branches  = 0;
}
      
/**
 * Count the number of moves having the same "pos" value. These
 * moves need to be resolved.
 */
int 
Board::count_ambiguous_forks (const Move &move) 
const
{
  int count = 0;
  for (int i=0; i<num_branches; i++) {
    if (move.position == moves[i].position) count++;
  }
  return count;
}
    
/**
 * Perform the given move. The move can be partial, e.g., the user has moved
 * her kas, but has yet to decide either to capture blacks or whites.
 *
 * full_move should be true for virtual players, since virtual players are not
 * constrained to use a sequential input device to make their moves.
 * full_move should also be true for remote players, since moves from remote players
 * are always marshalled.
 *
 * Unchecked precondition: if full_move==false, generate_branches should be called first,
 * and the branches must not be touched (sorted or singularized).
 */
void 
Board::perform (const Move& move, bool full_move)
{
  g_assert (stage != STAGE_NULL);
  switch (stage) {
  case STAGE_OPENING :
    first_move[player] = move.position;
    // nobreak
  case STAGE_RESOLVE_KAS_SELECTION :
  case STAGE_RESOLVE :
    perform_capture (move);
    change_player ();
    generate_branches ();
    break;
  case STAGE_SELECT_KAS :
  case STAGE_MOVE_KAS :
    perform_move_kas (move);
    if (!full_move && count_ambiguous_forks (move) > 1) {
      // Partial move. Needs another mouse click to decide capture.
      stage = (stage == STAGE_MOVE_KAS) ? STAGE_RESOLVE : STAGE_RESOLVE_KAS_SELECTION;
      generate_resolving_moves (move);
    }
    else {
      // Complete the move with a capture. Next player.
      perform_capture (move);
      change_player ();
      generate_branches ();
    }
    break;
  }
  
  if (count_branches () == 0) {
    stage = STAGE_GAME_OVER;
    if (value_of_uncaptured_pieces > 0) { // Out of moves (suntuk)
      score[1-player] += score[player];
      score[player] = 0;
    }
  }
}

/**
 * Perform the given text-encoded move (Move::to_string value).
 * Unchecked precondition: generate_branches should be called first, and the branches should
 * not be touched.
 */
void
Board::perform (const std::string& move_notation)
{
  for (int i=0; i < count_branches (); i++) {
    const Move& move = get_branch (i);
    if (move_notation == move.to_string ()) {
      perform (move, true);
      return;
    }
  }
  g_assert_not_reached ();
}
  
void
Board::perform_move_kas (const Move& move)
{
  const int position = move.position;
  if (stage == STAGE_SELECT_KAS) {
    // From move.position, a piece is moved onto kas_position[player]
    // to become the player's kas.
    int col = get_first_move (player);
    int row = position / MATRIX_WIDTH;
    kas_position[player] = col + row * MATRIX_WIDTH;
    // Since the promoted token is not capturable, its value is lost
    value_of_uncaptured_pieces -= (matrix[position] == PIECE_WHITE) ? 2 : 1;
    // Update the matrix
    matrix[position] = PIECE_EMPTY;
    matrix[kas_position[player]] = (PieceType) player;
  }
  else { 
    // Normal move
    matrix[kas_position[player]] = PIECE_EMPTY;
    matrix[position] = (PieceType) player;
    kas_position[player] = position;
  }
}

/**
 * Capture the pieces listed in the move's list of booties, making 
 * those positions empty. The player's score is updated.
 */
void 
Board::perform_capture (const Move& move)
{
  for (int n=0; n < move.count_booties (); n++) {
    const int pos = move.get_booty_position (n);
    if (matrix[pos] == PIECE_WHITE) {
      value_of_uncaptured_pieces -= 2;
      score[player] += 2;
    }
    else if (matrix[pos] == PIECE_BLACK) {
      value_of_uncaptured_pieces --;
      score[player] ++;
    }
    else {//if error
      for (int i=0; i < MATRIX_SIZE; i++) {
        if (i%MATRIX_WIDTH == 0) printf("\n");
        switch (matrix[i]) {
          case PIECE_WHITE : printf("2"); break;
          case PIECE_BLACK : printf("1"); break;
          case PIECE_KAS_0 : printf("R"); break;
          case PIECE_KAS_1 : printf("B"); break;
          case PIECE_EMPTY : printf("."); break;
          default          : printf(" "); break;
        }
      }
      printf ("\nid=%d pos=%d\n", move.id, move.position);
      g_assert (false && "Capturing uncapturable piece");
    }//endif error
    matrix[pos] = PIECE_EMPTY;
  }//endfor
}
    
/** 
 * Get the piece placed on the matrix at
 * the given row and col. The top-left corner is postion 1,1 (position 
 * 0,0 is occupied by a sentinnel token). Player 0 always sits at the "top".
 * The display routine has to rotate the board if Player 0 is visually
 * at the bottom.
 */
PieceType 
Board::get_piece (const int row, const int col) 
const
{
  return matrix[row*MATRIX_WIDTH + col];
}
    
/**
 * Get the token placed on the matrix at the pre-computed
 * position. (pos = row*MATRIX_WIDTH + col)
 */
PieceType 
Board::get_piece (const int pos)
const
{
  return matrix[pos];
}
    
    
/**
 * Get the "stage" of the game.
 */
GameStageType 
Board::get_stage ()
const
{
  return stage;
}
    
/**
 * Get the player to play. 0 means the first player,
 * 1 means the second player.
 */
int 
Board::get_player ()
const
{
  return player;
}

PieceType
Board::get_kas ()
const
{
  return (PieceType) player;
}

/**
 * Set up for the next player's turn, advancing the stage of the game.
 */
void 
Board::change_player ()
{
  if (stage == STAGE_RESOLVE_KAS_SELECTION)
    stage = STAGE_SELECT_KAS;
  else if (stage==STAGE_RESOLVE)
    stage = STAGE_MOVE_KAS;
    
  if (player == 1) {
    if (stage == STAGE_OPENING)
      stage = STAGE_SELECT_KAS;
    else if (stage == STAGE_SELECT_KAS)
      stage = STAGE_MOVE_KAS;
  }
  
  player = 1-player;
}

/**
 * Get the player's first move. This is needed (1) to make sure that
 * Player 1 does not copy Player 0's first move, (2) to generate the 
 * player's kas selection move, and (3) to determine onto which
 * column a kas candidate should be dropped.
 */
int 
Board::get_first_move (int p)
const
{
  g_assert(stage == STAGE_OPENING || stage == STAGE_SELECT_KAS);
  return first_move[p];
}
    
/**
 * Get the position of a player's kas.
 */
int 
Board::get_kas_position (int p)
const
{
  return kas_position[p];
}
    
/**
 * Get the player's score
 */
int 
Board::get_score (int p)
const
{
  return score[p];
}
    
/**
 * Start the game with the given starting pattern.
 * Player 0 always plays first and the stage of the game
 * always begins with <code>STAGE_OPENING</code>.
 */
void 
Board::start_game (const Matrix starting_matrix)
{
  value_of_uncaptured_pieces = 0;
  for (int n=0; n < MATRIX_SIZE; n++) {
    matrix[n] = starting_matrix[n];
    if (matrix[n] == PIECE_BLACK) 
      value_of_uncaptured_pieces += 1;
    else if (matrix[n] == PIECE_WHITE)
      value_of_uncaptured_pieces += 2;
  }
  g_assert (value_of_uncaptured_pieces == 60 + 60*2);     // 60 blacks && 60 whites
  player = 0;
  score[0] = 0;
  score[1] = 0;
  stage = STAGE_OPENING;
  generate_branches ();
}


/**
 * Move generation:
 */

/**
 * In the traditional way, prior to the game both players agree on how to make a
 * kas: either by sliding a piece adjacent to the empty column (previously created
 * by the opening move), or jumping over such a piece. 
 * In this version of pasang, the "agreement stage" is skipped. Each player is allowed
 * either option to select a kas. The code here recognizes all these three
 * modes (sliding, jumping, free) of kas selection. But the traditional modes are 
 * disabled. Only free play is available to the user.
 */
static const enum {
  SELECT_BY_JUMPING,
  SELECT_BY_SLIDING,
  SELECT_FREELY
} kas_selection = SELECT_FREELY;

/**
 * Generate a list of all possible moves.
 * Each move is given an ID.
 */
void
Board::generate_branches ()
{
  g_assert (stage != STAGE_NULL);
  num_branches = 0;
  if (stage == STAGE_OPENING) {
    generate_opening_moves ();
  }
  else if (stage == STAGE_SELECT_KAS) {
    if (kas_selection == SELECT_BY_JUMPING || kas_selection == SELECT_FREELY)
      generate_kas_selection_moves (2);
    if (kas_selection == SELECT_BY_SLIDING || kas_selection == SELECT_FREELY)
      generate_kas_selection_moves (1);
  }
  else if (stage==STAGE_MOVE_KAS) {
    generate_kas_moving_moves ();
  }
  else {
    g_assert (false && "No move for the given stage");
  }
}

/**
 * "Generate" resolving moves. Actually, there are no moves to generate.
 * We just pick the moves that were previously generated. 
 *
 * The argument is a representative move where the user has yet to decide 
 * the capture (either blacks or whites). This method picks all moves 
 * (including the argument) that has the same pos (but different booty).
 *
 * Assumptions: 1) moves with the same pos are listed in a run.
 *              2) the argument is the first move within the run.
 *              3) moves[x].id = x
 * (All these assumptions are fulfilled by generate_branches, 
 * and nullified by generate_resolving_moves, singularize_branch and sort_branches)
 */
void 
Board::generate_resolving_moves (const Move& move)
{
  const int id = move.id;
  const int position = move.position;
  for (int i=0; i < num_branches; i++) {
    if (id+1 < num_branches && moves[id+i].position == position) {
      moves[i] = moves[id+i];
    }
    else {
      g_assert (i>=2);  // If the capture is unambigious, this method shouldn't be called.
      g_assert (i<=3);  // Capture blacks, capture whites, up-capture in kas creation. No more.
      num_branches = i;
      break;
    }
  }
}
  
/**
 * Make this a list of one move only
 */
void 
Board::singularize_branch (Move& m)
{
  if (moves[0].id != m.id) // Test added to silence valgrind "src and dest overlap in memcpy"
    moves[0] = m;
  moves[0].id = 0;
  num_branches = 1;
}

/**
 * The number of moves generated
 */
int 
Board::count_branches ()
const
{
  return num_branches;
}

/**
 * Sort generated moves in descending score
 */
int 
cmpMoves (const void* move1, const void* move2)
{
  return ((Move*)move2)->score - ((Move*)move1)->score;
}

void 
Board::sort_branches ()
{
  qsort (moves, num_branches, sizeof(Move), cmpMoves);
}

/**
 * Retrieve the nth move
 */
Move& 
Board::get_branch (int n)
{
  g_assert (n >= 0 && n < num_branches);
  return moves[n];
}

void 
Board::generate_opening_moves ()
{
  // Recall player 0's first move. Make sure that player 1 does not copy this move.
  const int forbidden_column = (player == 1) ? (12 - get_first_move (0)) : -1;
  // Player 0 may capture these 5 pieces: (x,1), (x,2), (x,3), (x,4), (x,5)
  // Player 1 may capture these 5 pieces: (x,7), (x,8), (x,9), (x,10), (x,11)
  // where x element {1,2,...,11}-{forbidden_column} is the grid column, 
  // and the numbers are the grid rows.
  // ref_pos refers to either grid (0,1) or grid (0,7), depending on the player to move.
  const int ref_pos =  ((player == 0) ? 1 : 7) * MATRIX_WIDTH;
  for (int col=1; col <= 11; col++) {
    if (col == forbidden_column) continue;
    add_move(col, 5, ref_pos + col, MATRIX_WIDTH);
  }
}

/**
 * The kas can be selected from a piece on the banks, each two-column wide, astriding 
 * the empty column that was captured during the opening move. Such a piece is either 
 * one grid or two grid away from the empty column (see figure below where O represent 
 * a candidate kas).
 *
 *                 OOeOO
 *                 OOmOO
 *                 OOpOO
 *                 OOtOO
 *                 OOyOO
 *
 * generate_kas_selection_moves (1 [or 2]) choose candidates 1 [or 2] grid away from 
 * the empty column.
 */
void 
Board::generate_kas_selection_moves (int distance)
{
  g_assert (distance==1 || distance==2);
  const int row = (player == 0) ? 1 : 7;
  const int col = get_first_move (player);

  // For each of the 5 grid points in the empty column, starting from the top
  for (int j=0,  empty_position = row * MATRIX_WIDTH + col; 
       j < 5; 
       j++,  empty_position += MATRIX_WIDTH) {
       
    // <diversion>
    //   Note that the test "candidate!=PIECE_EMPTY" below was missing from the original
    //   Pasang Emas (non-free, and this bug will live forever in there). As a result, 
    //   a captured piece can still be used for the kas!. (e.g. Select galap pattern. 
    //   Player 0 capture column 3, Player 1 capture column 2. Now Player 0 can 
    //   capture the three black pieces, two of which are player 1's potential kas).
    // </diversion>
    
    // Consider the candidate on the "negative" side of the empty column
    if (col > distance) {  // Don't go outside the board
      const int candidate_position = empty_position - distance;
      const PieceType candidate = matrix[candidate_position];
      if (candidate != PIECE_EMPTY) {
        // Lift the candidate
        matrix[candidate_position] = PIECE_EMPTY;
        // See if it can capture any pieces horizontally
        add_move (candidate_position, empty_position, 1);
        // ... and vertically
        add_move (candidate_position, empty_position, MATRIX_WIDTH);
        // Put back the candidate
        matrix[candidate_position] = candidate;
      }
    }
    // Do the same thing to the "positive" side of the empty column
    if (col < 12 - distance) {
      const int candidate_position = empty_position + distance;
      const PieceType candidate = matrix[candidate_position];
      if (candidate != PIECE_EMPTY) {
        matrix[candidate_position] = PIECE_EMPTY;
        add_move (candidate_position, empty_position, 1);
        add_move (candidate_position, empty_position, MATRIX_WIDTH);
        matrix[candidate_position] = candidate;
      }
    }
  }//end for j
}

/**
 * The kas can move 
 *   up (direction: -MATRIX_WIDTH), capturing horizontally (direction: 1),
 *   down (direction: +MATRIX_WIDTH), capturing horizontally (direction: 1),
 *   left (direction: -1), capturing vertically (direction: MATRIX_WIDTH),
 *   right (direction: +1), capturing vertically (direction: MATRIX_WIDTH).
 */  
void 
Board::generate_kas_moving_moves ()
{
  int kas_directions[]     = {-MATRIX_WIDTH, MATRIX_WIDTH, -1, 1};
  int capture_directions[] = {1, 1, MATRIX_WIDTH, MATRIX_WIDTH};
  for (int i=0; i < 4; i++) {
    int position = kas_position[player] + kas_directions[i];
    while (matrix[position] == PIECE_EMPTY) {
      add_move (position, position, capture_directions[i]);
      position += kas_directions[i];
    }
  }
}
    
/**
 * Construct a move for each possible capture in the given capture_direction
 * (1 for horizontal capture, MATRIX_WIDTH for vertical capture). Only an odd 
 * number of pieces of the same colour can be captured. These pieces must be
 * in line with capture_position.
 *
 * The position parameter is used for Move::position.
 */
void 
Board::add_move (int position, int capture_position, int capture_direction)
{
  g_assert (capture_direction == 1 || capture_direction == MATRIX_WIDTH);
  int num_booties[2];
  int scan_positions[2];
  PieceType booties[2];
  // After the following loop, these are set:
  //   num_booties:       numbers of pieces of the same colour
  //   booties:           the first non-empty pieces found
  //   scan_positions[0]: position of the left-most (or top-most) booty/empty
  //   scan_positions[1]: position of the right-most (or deepest) booty/empty
  for (int i=0; i < 2; i++) {
    // Direction to scan for booties:         up/left       or     down/right
    const int scan_direction = (i == 0) ? -capture_direction : capture_direction;
    // Find the first booty below in the scan_direction
    scan_positions[i] = capture_position + scan_direction;
    while (matrix[ scan_positions[i] ] == PIECE_EMPTY) {
      scan_positions[i] += scan_direction;
    }
    booties[i] = matrix[scan_positions[i]];
    // Count booties in the scan_direction
    num_booties[i] = 0;
    if (booties[i] == PIECE_WHITE || booties[i] == PIECE_BLACK) {
      num_booties[i] = 1;
      for (;;) {
        scan_positions[i] += scan_direction;
        const PieceType piece = matrix[ scan_positions[i] ];
        if (piece == PIECE_EMPTY) continue;
        if (piece != booties[i]) break;
        num_booties[i] ++;
      }
      scan_positions[i] -= scan_direction;  // Point back to the last booty/empty
    }
  }//endfor i
        
  if (booties[0] == booties[1]) {
    // Only one color can be captured
    const int sum_booties = num_booties[0] + num_booties[1];
    if (sum_booties&1)    // Capture odd number of pieces only
      add_move (position, sum_booties, scan_positions[0], capture_direction);
  }
  else {
    // Fork: capture either black or white
    if (num_booties[0]&1) // Capture odd number of pieces only
      add_move (position, num_booties[0], scan_positions[0], capture_direction);
    if (num_booties[1]&1) // Capture odd number of pieces only
      add_move (position, num_booties[1], scan_positions[1], -capture_direction);
  }
}
    
/**
 * Construct a move. Grab the required number of pieces. No checking is 
 * performed for piece type uniformity. Just grab consecutive pieces,
 * either black or white or mixed.
 */
void 
Board::add_move (int position, int num_booties, int capture_position, int capture_direction)
{
  g_assert (num_branches < MAX_BRANCH);
  g_assert(num_booties&1);
  moves[num_branches].set (stage, player, num_branches, position);
  for (int n=0;  n < num_booties;  capture_position += capture_direction) {
    const PieceType piece = matrix[capture_position];
    if (piece != PIECE_EMPTY) {
      g_assert (piece == PIECE_BLACK || piece == PIECE_WHITE);
      moves[num_branches].add_booty (capture_position, piece);
      n++;
    }
  }
	num_branches++;
}

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
