/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "brain.h"
using namespace Pasang;

/**
 * The brain thinks using its own thread so that the GUI remains
 * responsive.
 */
Brain::Brain ()
{
  exiting = false;
  stopping = false;
  thinking = false;
  thinking_thread = Glib::Thread::create (sigc::mem_fun (*this, &Brain::run), true);
}

Brain::~Brain ()
{
  request_exit();
}

/**
 * Request the thinking thread to exit. Wait for the
 * request to be complied.
 */
void 
Brain::request_exit ()
{
  exiting = true;
  stopping = true;
  {Glib::Mutex::Lock lock (mutex);
    thinking = false;
    cond.signal();
  }
  thinking_thread->join ();
}

/**
 * Request the thinking thread to abandon on-going thinking. 
 * Wait for the request to be complied.
 */
void 
Brain::request_stop ()
{
  stopping = true;
  {Glib::Mutex::Lock lock (mutex);
    stopping = false;
  }
}

/**
 * Request thinking on a new board configuration, killing any
 * on-going thinking prior to that.
 */
void 
Brain::request_think (Board& board, long id, VirtualPlayer vp)
{
  g_assert (board.get_stage () != STAGE_GAME_OVER);
  stopping = true;
  {Glib::Mutex::Lock lock (mutex);
    stopping = false;
    thinking = true;
    ai_board = Board (board);
    request_id = id;
    virtual_player = vp;
    cond.signal();
  }
}

/**
 * The task for the thinking thread. To stop this thread, call request_exit.
 */
void
Brain::run ()
{
  while (! exiting) {
    {Glib::Mutex::Lock lock (mutex);
      if (thinking) {
        response_id = request_id;  // emph: request_id is volatile
        response = think ();
        if (!stopping) got_move_signal ();
        thinking = false;
      }
      if (!exiting) cond.wait (mutex);
    }//endlock
  }//endwhile
}
     
/**
 * Perform iterative deepening, and then inject some randomness to 
 * the move selection.
 */
Move* 
Brain::think ()
{
  // Initialise according to player's characteristics
  const long max_time = virtual_player.thinking_time;
  const int  max_ply  = virtual_player.thinking_depth * 2;
        
  // Initialise the root node
  ai_board.generate_branches ();
  const int num_moves = ai_board.count_branches ();
  if (num_moves == 1) return &(ai_board.get_branch (0));  // The only move
  for (int n=0; n < num_moves; n++) {
    ai_board.get_branch(n).score = NEGATIVE_CUT_OFF;
  }
        
  // Perform iterative-deepening
  timer.reset ();
  for (int depth=1; depth < max_ply; depth++) {
    // Interrupted or out of time?
    if (stopping || timer.elapsed () > max_time) break;
    bool depth_reached = false;
    int  nodes = 0;
    search (0, depth, max_time, &depth_reached, &nodes,
            ai_board, NEGATIVE_CUT_OFF, NEGATIVE_CUT_OFF);
    // For better cut-off, sort the root branches.
    // Sorting is also assumed by roll_biased_dice()
    ai_board.sort_branches ();
    // If the previous search iteration did not need 
    // that many plies, we are done.
    if (!depth_reached) break;
  }

  return &(ai_board.get_branch (roll_biased_dice ()));
}
    
/**
 * Search to the given depth. Return evaluation.
 * Side effect: each move of board evaluated.
 *              *depth_reached set to true if ply == depth.
 *              *nodes set to the number of nodes generated.
 */
int
Brain::search (int ply, int depth, long max_time, bool *depth_reached, int *nodes,
               Board &board, int my_cut_off, int opponent_cut_off)
{
  if (ply == depth) *depth_reached = true;
  int player = board.get_player ();

  int best_value = NEGATIVE_CUT_OFF;
  // Cutoff value for deep alpha-beta pruning.
  // The position at ply 1 is FULLY evaluated. 
  // (The following line is not really required if pruning is exhaustive).
  if (ply < 3) my_cut_off = NEGATIVE_CUT_OFF;
  // If game over or max depth is reached, backtrack
  if (board.get_stage () == STAGE_GAME_OVER  ||  ply == depth) {
    return (board.get_score (player) - board.get_score (player ^ 1));
  }

  int num_moves = board.count_branches ();
  for (int i=0; i < num_moves; i++) {
    // Get the next move, apply, and search deeper
    Move& move = board.get_branch (i);
    Board board2 = Board (board);
    board2.perform (move, true);  // Generate moves
    int value = - search (ply + 1, depth, max_time, depth_reached, nodes,
                          board2, opponent_cut_off, my_cut_off);
    (*nodes)++;
    
    // Once in a while, check events
    if (stopping || (*nodes & 1024) && timer.elapsed () > max_time) {
      break;
    }
    
    // Hoist the result up
    board.get_branch(i).score = value;
    if (value > best_value) {
      best_value = value;
      my_cut_off = value;     // This line added in version 1.1.2
      // Perform alpha-beta pruning. The positions at ply 1 are FULLY 
      // evaluated (hence the condition ply>1 rather than ply>0). This is
      // done so that the random selection of moves will be based on the 
      // actual value of each move (rather than the "at-most" value when 
      // cut-off is applied).
      if (ply > 1 && best_value >= - opponent_cut_off) {
        break;
      }
      // Prefer quickest win. (not implemented yet)
      // if (win by suntuk) break;
    }
  }//end for
  return best_value;
}

    
/**
 * This method provides some variability in the line of play.
 * Assume ai_board's branches are sorted in descending scores
 */
int 
Brain::roll_biased_dice ()
{
  const int num_moves = ai_board.count_branches ();
  // Add further discernment to contrast good and bad moves
  const int discernment = (virtual_player.discernment - 1) * 2;
  for (int i=0; i < num_moves; i++) {
    int& score_ref = ai_board.get_branch(i).score;
    if (score_ref > 0) score_ref += discernment;
  }
  // "Translate" the scores so that the smallest one is 1.
  // Calculate the total of the translated scores.
  const int smallest_score = ai_board.get_branch(num_moves - 1).score;
  int total = 0;
  for (int i=0; i < num_moves; i++) {
    total += ai_board.get_branch(i).score - smallest_score + 1;
  }
  // Roll the dice
  int num_rolls = virtual_player.discernment;
  // In the early stage of the game, prefer variety rather than consistency
  if (ai_board.get_stage () == STAGE_OPENING) num_rolls = 1;
  else if (ai_board.get_stage () == STAGE_SELECT_KAS) num_rolls /= 2;
  int roll = rand.get_int_range (0, total);
  for (int i=1; i < num_rolls; i++) {
    int roll2 = rand.get_int_range (0, total);
    if (roll > roll2) roll = roll2;
  }
  // Look at the move selected by the best roll
  for (int i=0; i < num_moves; i++) {
    roll -= ai_board.get_branch(i).score - smallest_score + 1;
    if (roll < 0) return i;
  }
  return 0;
}

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
