/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <gtkmm/stock.h>
#include <gtkmm/table.h>
#include <gtkmm/scale.h>
#include <gtkmm/label.h>
#include <gtkmm/separator.h>
#include "brainsetter.h"
#include "ucompose.hpp"
using namespace Pasang;

/** 
 * Controls
 */
static struct {
  const char   *range_label;
  const char   *min_label;
  const char   *max_label;
  double  min; 
  double  max;
} controls[] = {
  { N_("Thinking Time in Seconds:"),
    // fast: the computer must finish thinking within 1 second
    N_("fast"), 
    // slow: the computer can take up to 10 seconds to think
    N_("slow"),
    1, MAX_TIME
  },
  { N_("Number of Moves to Think Ahead:"),
    // shallow: the computer thinks by looking one move ahead
    N_("shallow"),
    // deep: the computer thinks by looking 10 moves ahead
    N_("deep"),
    1, MAX_MOVE
  },
  { N_("Ability to Discern Good Positions:"),
    // fuzzy: the computer does not always choose the best moves
    N_("fuzzy"), 
    // sharp: the computer prefers the best moves
    N_("sharp"),
    1, MAX_DISCERNMENT
  }
};

/**
 * Preset virtual characters
 */
static struct {
  const char *name;
  int  thinking_time;
  int  thinking_depth;
  int  discernment;
} brain_settings[] = {
  // Cakoi: fried garlic bread. Please don't translate. Leave it, or transliterate.
  { N_("Cakoi"),     1,  1,  1 },
  // Ambuyat: gummy sago paste. Please don't translate. Leave it, or transliterate.
  { N_("Ambuyat"),   1,  2,  2 },
  // Belacan: fermented shrimp paste. Please don't translate. Leave it, or transliterate.
  { N_("Belacan"),   2,  4,  4 },
  // Cincaluk: fermented shrimp dip. Please don't translate. Leave it, or transliterate.
  { N_("Cincaluk"),  4,  5,  5 },
  // Binjai: a kind of fruit. Please don't translate. Leave it, or transliterate.
  { N_("Binjai"),    1,  2,  MAX_DISCERNMENT },
  // Piasau: coconut. Please don't translate. Leave it, or transliterate.
  { N_("Piasau"),    2,  MAX_MOVE, MAX_DISCERNMENT },
  // Kambayau: a kind of fruit. Please don't translate. Leave it, or transliterate.
  { N_("Kambayau"),  6,  MAX_MOVE, MAX_DISCERNMENT },
  { NULL }
};

BrainSetter::BrainSetter ()
{
  dialog = NULL;
  auto_adjusting = false;
}

BrainSetter::~BrainSetter ()
{
  if (dialog != NULL) delete dialog;
}


bool 
BrainSetter::show_dialog (char *title, Gtk::Window& parent, VirtualPlayer& virtual_player)
{
  if (dialog == NULL) make_dialog ();
  dialog->set_transient_for (parent);
  dialog->set_title (title);
  // Copy the characteristics of virtual_player to visual controls
  auto_adjusting = true;
  adjustments[INDEX_TIME_ADJUSTMENT]->set_value (virtual_player.thinking_time);
  adjustments[INDEX_DEPTH_ADJUSTMENT]->set_value (virtual_player.thinking_depth);
  adjustments[INDEX_DISCERNMENT_ADJUSTMENT]->set_value (virtual_player.discernment);
  auto_adjusting = false;
  on_value_changed ();
  // Wait for user's selection
  int result = dialog->run ();
  dialog->hide();
  if (result != Gtk::RESPONSE_OK) return false;
  // Change the characteristics of virtual_player 
  // based on user changes to the visual controls
  virtual_player.thinking_time  = (int) adjustments[INDEX_TIME_ADJUSTMENT]->get_value ();
  virtual_player.thinking_depth = (int) adjustments[INDEX_DEPTH_ADJUSTMENT]->get_value ();
  virtual_player.discernment    = (int) adjustments[INDEX_DISCERNMENT_ADJUSTMENT]->get_value ();
  return true;
}

Glib::ustring
BrainSetter::get_name (VirtualPlayer& virtual_player)
{
  // See if virtual_player matches any of the preset settings
  for (int i=0; brain_settings[i].name; i++) {
    if (virtual_player.thinking_time == brain_settings[i].thinking_time
        && virtual_player.thinking_depth == brain_settings[i].thinking_depth
        && virtual_player.discernment == brain_settings[i].discernment) {
      return _(brain_settings[i].name);
    }
  }
  // Otherwise, simply give the spec
  return String::ucompose ("%1-%2-%3",
    virtual_player.thinking_time, 
    virtual_player.thinking_depth,
    virtual_player.discernment);
}

void 
BrainSetter::make_dialog ()
{
  const int padding = 5;
  
  dialog = new Gtk::Dialog ("", true, true);
  dialog->set_resizable (false);
  Gtk::VBox* vbox = dialog->get_vbox ();

  // Table for laying out Adjustments
  Gtk::Table* table = Gtk::manage (new Gtk::Table (6, 1, false));
  //table->set_row_spacings(10);
  vbox->pack_start (*table, Gtk::SHRINK, 5);

  // List out preset brains
  name_list = Gtk::manage (new Gtk::ComboBoxText ());
  name_list->signal_changed ().connect (sigc::mem_fun (*this, &BrainSetter::on_name_changed));
  for (int i=0; brain_settings[i].name; i++) {
    name_list->append_text ( _(brain_settings[i].name));
  }
  table->attach (*name_list, 0, 1, 0, 1, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, padding);
  table->attach (*(Gtk::manage (new Gtk::HSeparator ())), 0, 1, 1, 2);

  // Construct Adjustments and their views
  for (int i=0; i < NUM_ADJUSTMENTS; i++) {
    adjustments[i] = Gtk::manage (new Gtk::Adjustment (
                        controls[i].min, controls[i].min, controls[i].max));
    adjustments[i]->set_page_increment (1);
    adjustments[i]->signal_value_changed ().connect (
      sigc::mem_fun (*this, &BrainSetter::on_value_changed));

    Gtk::HScale* scale = Gtk::manage (new Gtk::HScale (*(adjustments[i])));
    scale->set_digits (0);
    scale->set_size_request (300);
    scale->set_draw_value (true);
    scale->set_value_pos (Gtk::POS_BOTTOM);
    scale->signal_format_value ().connect (
      sigc::bind (sigc::mem_fun (*this, &BrainSetter::tick_label), i));

    Gtk::Label* label = 
      Gtk::manage (new Gtk::Label ( _(controls[i].range_label), Gtk::ALIGN_LEFT));
    
    Gtk::VBox* box = Gtk::manage (new Gtk::VBox ());
    box->pack_start (*label);
    box->pack_start (*scale);
    table->attach (*box, 0, 1, i+2, i+3, 
                   Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, padding, padding);
  }//endfor i
  
  dialog->add_button (Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog->add_button (Gtk::Stock::OK, Gtk::RESPONSE_OK);
  dialog->show_all_children ();
}

Glib::ustring 
BrainSetter::tick_label (double _value, int id)
{
  int value = (int) _value;
  if (value == controls[id].min) return _(controls[id].min_label);
  if (value == controls[id].max) return _(controls[id].max_label);
  return "";
}

void 
BrainSetter::on_value_changed ()
{
  int  thinking_time  = (int)adjustments[INDEX_TIME_ADJUSTMENT]->get_value ();
  int  thinking_depth = (int)adjustments[INDEX_DEPTH_ADJUSTMENT]->get_value ();
  int  discernment   = (int)adjustments[INDEX_DISCERNMENT_ADJUSTMENT]->get_value ();
  for (int i=0; brain_settings[i].name; i++) {
    if (thinking_time == brain_settings[i].thinking_time
        && thinking_depth == brain_settings[i].thinking_depth
        && discernment == brain_settings[i].discernment) {
      auto_adjusting = true;
      name_list->set_active (i);
      name_list->add_label (name_list->get_active_text ());
      auto_adjusting = false;
      return;
    }
  }
  name_list->set_active (-1);
  name_list->add_label (String::ucompose ("%1-%2-%3", 
    thinking_time, thinking_depth, discernment));
}

void 
BrainSetter::on_name_changed ()
{
  if (auto_adjusting) return;
  int row = name_list->get_active_row_number ();
  if (row != -1) {
    auto_adjusting = true;
    adjustments[INDEX_TIME_ADJUSTMENT]->set_value (brain_settings[row].thinking_time);
    adjustments[INDEX_DEPTH_ADJUSTMENT]->set_value (brain_settings[row].thinking_depth);
    adjustments[INDEX_DISCERNMENT_ADJUSTMENT]->set_value (brain_settings[row].discernment);
    auto_adjusting = false;
    name_list->add_label (name_list->get_active_text ());
  }
}

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
