/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "theme.h"
#include "inline-images.h"
#include "ucompose.hpp"
#include <glibmm/fileutils.h>
#include <gtkmm/stock.h>
#include <gtkmm/scrolledwindow.h>

using namespace Pasang;

Theme::Theme ()
{
  create_dialog ();
  current_theme = -1;
  requested_theme = 0;
  current_zoom = -1;
  requested_zoom = 100;
  change ();
}


Theme::~Theme ()
{
  delete dialog;
}


void 
Theme::draw_board (Glib::RefPtr<Gdk::Window> win, Glib::RefPtr<Gdk::GC> gc, 
                   int x, int y)
{
  board_image->render_to_drawable
    (win, gc,
    0, 0, x, y,
    board_image_width (), board_image_width (),
    Gdk::RGB_DITHER_NONE, 0, 0);
}


void 
Theme::draw_piece (Glib::RefPtr<Gdk::Window> win, Glib::RefPtr<Gdk::GC> gc, 
                   PieceType pt, int frame, int x, int y)
{
  g_assert ((int) pt >= 0 && pt < PIECE_COUNT);
  frame = MIN (frame, count_frames (pt) - 1);
  int width = piece_image_width (pt);
  piece_image[pt]->render_to_drawable 
    (win, gc,
    frame * width, 0, x, y, 
    width, width,
    Gdk::RGB_DITHER_NONE, 0, 0);
}


bool
Theme::detect_piece (PieceType pt, int frame, int x, int y)
{
  int width = piece_image_width (pt);
  x += width / 2;
  y += width / 2;
  frame = MIN (frame, count_frames (pt) - 1);
  guint8* rgba = piece_image[pt]->get_pixels ()
                 + 4 * (y * piece_image[pt]->get_width () 
                        + frame * width + x);
  return (rgba[0] | rgba[1] | rgba[2] | rgba[3]) != 0;
}


int
Theme::count_frames (PieceType pt)
{
  return piece_image[pt]->get_width () / piece_image[pt]->get_height ();
}


int
Theme::board_image_width ()
{
  return board_image->get_width ();
}


int
Theme::piece_image_width (PieceType pt)
{
  return piece_image[pt]->get_height ();
}


int
Theme::cell_width ()
{
  return board_image_width () / MATRIX_WIDTH;
}


void
Theme::show_dialog (Gtk::Window &parent) 
{
  dialog->set_transient_for (parent);
  dialog->show ();
}


void
Theme::create_dialog ()
{
  dialog = new Gtk::Dialog (_("Choose Theme"), false, false);

  Gtk::VBox* vbox = dialog->get_vbox ();
  vbox->set_size_request (200, 300);
  vbox->add (*create_theme_list ());
  
  //dialog->add_button (Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);
  dialog->show_all_children ();
}

    
Gtk::Widget*
Theme::create_theme_list ()
{
  // Construct model and view for selectable theme list
  Gtk::TreeModelColumn<Glib::ustring> *name      = new Gtk::TreeModelColumn<Glib::ustring>;
  Gtk::TreeModel::ColumnRecord        *rec_model = new Gtk::TreeModel::ColumnRecord;
  rec_model->add (*name);
  tree_model = Gtk::ListStore::create (*rec_model);
  tree_view  = Gtk::manage (new Gtk::TreeView);
  tree_view->set_model (tree_model);

  // Assemble theme list
  // (1) built-in theme
  num_themes = 1;
  Gtk::TreeModel::Row row = *(tree_model->append ());
  row[*name] = Glib::ustring ("kayu");
  theme_dirs.push_back ("kayu");
  // (2) themes found in $datadir/pasang-emas
  try {
    Glib::Dir theme_dir (THEME_ROOT_DIR);
    while (true) {
      std::string dir = theme_dir.read_name ();
      if (dir == "") break;
      Glib::ustring path = Glib::build_filename (THEME_ROOT_DIR, dir);
      if (Glib::file_test (path, Glib::FILE_TEST_IS_DIR)) {
        row = *(tree_model->append ());
        row[*name] = dir;
        theme_dirs.push_back (path);
        num_themes++;
      }
    }
  }
  catch (Glib::Error e) {
    printf("File error: %s\n", e.what().c_str());
  }
  // (3) themes found in $home/.pasang-emas
  std::string theme_dir_name = Glib::build_filename (Glib::get_home_dir (), USER_THEME_ROOT_DIR);
  try {
    Glib::Dir user_theme_dir (theme_dir_name);
    while (true) {
      std::string dir = user_theme_dir.read_name ();
      if (dir == "") break;
      Glib::ustring path = Glib::build_filename (theme_dir_name, dir);
      if (Glib::file_test (path, Glib::FILE_TEST_IS_DIR)) {
        row = *(tree_model->append ());
        row[*name] = dir;
        theme_dirs.push_back (path);
        num_themes++;
      }
    }
  }
  catch (Glib::Error e) {
    printf("File error: %s\n", e.what().c_str());
  }
  tree_view->append_column ("Name", *name);
  

  // Register listener for change in pattern selection
  tree_selection = tree_view->get_selection ();
  tree_selection->signal_changed().connect (sigc::mem_fun(*this, &Theme::on_selection));

  // Set up visual
  tree_view->columns_autosize ();
  tree_view->set_headers_visible (false);
  tree_view->set_cursor (Gtk::TreePath ("0"));
  Gtk::ScrolledWindow* scrolled = Gtk::manage (new Gtk::ScrolledWindow);
  scrolled->set_policy (Gtk::POLICY_NEVER, Gtk::POLICY_ALWAYS);
  scrolled->add (*tree_view);
  
  delete name;
  delete rec_model;
  return scrolled;
}


void
Theme::on_selection ()
{
  Glib::Mutex::Lock lock (mutex);
  Gtk::TreeModel::iterator iter = tree_selection->get_selected ();
  if (!iter) return;
  Gtk::TreeModel::Row row = *iter;
  Glib::ustring theme_num = tree_model->get_string (iter);
  requested_theme = atoi (theme_num.c_str ());
}


void 
Theme::random ()
{
  int rnd = rand.get_int_range (0, num_themes);
  Glib::ustring path = String::ucompose ("%1", rnd);
  tree_selection->unselect_all ();
  tree_view->scroll_to_row (Gtk::TreePath (path));
  tree_selection->select (Gtk::TreePath (path));
}


void
Theme::zoom (int z)
{
  Glib::Mutex::Lock lock (mutex);
  requested_zoom = z;
}


bool
Theme::change ()
{
  Glib::Mutex::Lock lock (mutex);
  bool b1 = change_theme ();
  bool b2 = change_zoom ();
  return b1 || b2;  // b1 and b2 must be fully evaluated
}

bool
Theme::change_theme ()
{
  if (requested_theme == current_theme) return false;
  current_theme = requested_theme;
  current_zoom = -1;
  
  if (current_theme != 0) {
    std::string theme_dir = theme_dirs[ current_theme ];
    try {
      const char* filenames[] = {"red.png", "blue.png", "black.png", "white.png", "board.png", NULL};
      for (int i=0; filenames[i]; i++) {
        images[i] = Gdk::Pixbuf::create_from_file (Glib::build_filename (theme_dir, filenames[i]));
      }
    }
    catch (Glib::Error e) {
      current_theme = 0;
      requested_theme = 0;
      printf ("File error: %s\n", e.what().c_str());
    }
  }
  // sic: no else if below
  if (current_theme == 0) {
    const guint8* inline_images[] = {inline_image_red, inline_image_blue, 
                                     inline_image_black, inline_image_white, 
                                     inline_image_board, NULL};
    for (int i=0; inline_images[i]; i++) {
      try {
        images[i] = Gdk::Pixbuf::create_from_inline (-1, inline_images[i]);
      }
      catch (Glib::Error e) {
        printf ("Memory error: %s\n", e.what().c_str());
        g_assert (false);
      }
    }
  }
  return true;
}


bool
Theme::change_zoom ()
{
  if (requested_zoom == current_zoom) return false;
  current_zoom = requested_zoom;

  // Scale the board so that it is evenly divisible by the zoomed cell size
  int width = images[PIECE_COUNT]->get_width () / MATRIX_WIDTH * current_zoom / 100 * MATRIX_WIDTH;
  board_image = images[PIECE_COUNT]->scale_simple (width, width, Gdk::INTERP_BILINEAR);
  
  // Scale pieces so that the animation strip is evenly divisible by the zoomed frame size
  for (int i=0; i < PIECE_COUNT; i++) {
    int num_frames = images[i]->get_width () / images[i]->get_height ();
    int height = images[i]->get_height () * current_zoom / 100;
    width = height * num_frames;
    piece_image[i] = images[i]->scale_simple(width, height, Gdk::INTERP_BILINEAR);
  }
  
  kas_image[0] = Gdk::Pixbuf::create_subpixbuf
    (piece_image[PIECE_KAS_0], 0, 0, 
     piece_image_width (PIECE_KAS_0), piece_image_width (PIECE_KAS_0));
  kas_image[1] = Gdk::Pixbuf::create_subpixbuf 
    (piece_image[PIECE_KAS_1], 0, 0, 
     piece_image_width (PIECE_KAS_1), piece_image_width (PIECE_KAS_1));
  return true;
}
// vim: tabstop=2: expandtab: textwidth=100: autoindent:
