/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <gtkmm/stock.h>
#include <gtkmm/buttonbox.h>
#include <gtkmm/table.h>
#include <gtkmm/menubar.h>
#include <gtkmm/toolbar.h>
//#include <gtkmm/statusbar.h>
#include <gtkmm/aboutdialog.h>
#include <glibmm/timer.h>
#include <libgnome/gnome-help.h>
#include "window.h"
#include "board-gui.h"
#include "board.h"
#include "theme.h"
#include "pattern.h"
#include "ucompose.hpp"
using namespace Pasang;

enum ActionID {      // Signals for user action
  ACTION_NONE,
  ACTION_INIT,       // Not a user action
  ACTION_PASS,       // Let the computer move first
  ACTION_NEW_GAME,
  ACTION_NEXT_ROUND,
  ACTION_DEMO,
  ACTION_COPY,
  ACTION_UNDO,
  ACTION_QUIT,
  ACTION_THEME,
  ACTION_ZOOM100,
  ACTION_ZOOM50,
  ACTION_ZOOM75,
  ACTION_ABOUT,
  ACTION_HOW_TO_PLAY,
  TIMEOUT_ANIMATION,
  TIMEOUT_GAME_OVER,
  TIMEOUT_GAME_START
};

Window::Window ()
{
  //set_title (_(PACKAGE_NAME));  // This doesn't go to .pot file
  set_title (_("Pasang Emas"));
  try {
    set_icon_from_file (PIXMAPSDIR"/pasang-emas.png");
  }
  catch (Glib::Error e) {
    printf("File error: %s\n", e.what().c_str());
  }
  brain = new Brain;
  brain_setter = new BrainSetter;
  request_id = 0;
  recording = new Recording;
  demo_dialog = NULL;
  about_dialog = NULL;

  // Layout panels
  Gtk::VBox *main_panel = Gtk::manage (new Gtk::VBox);
  Gtk::HBox *big_panel  = Gtk::manage (new Gtk::HBox);
  add (*main_panel);
  
  // Menus, statusbar and toolbars
  create_menus ();
  Gtk::MenuBar* menu_bar = (Gtk::MenuBar*) (ui_manager->get_widget ("/MenuBar"));
  main_panel->pack_start (*menu_bar, Gtk::PACK_SHRINK, 0);
  //Gtk::Toolbar* tool_bar = (Gtk::Toolbar*) (ui_manager->get_widget ("/Toolbar"));
  //main_panel->pack_start (*tool_bar, Gtk::PACK_SHRINK, 0);
  statusbar = Gtk::manage (new Gtk::Label);
  statusbar->set_alignment (Gtk::ALIGN_LEFT);
  main_panel->pack_end (*statusbar);

  // Side panel for pattern selection
  main_panel->pack_start (*big_panel, Gtk::PACK_EXPAND_WIDGET, 0);
  pattern_selector = Gtk::manage(new PatternSelector);
  big_panel->pack_start (*pattern_selector, false, false);
  
  // Game board panel
  board_gui = Gtk::manage (new BoardGui);
  big_panel->pack_start (*board_gui, Gtk::PACK_EXPAND_WIDGET, 0);
  
  // Side panel
  Gtk::VBox *side_panel = Gtk::manage(new Gtk::VBox);
  big_panel->pack_start (*side_panel, Gtk::PACK_SHRINK, 5);

  kas_images[0]  = Gtk::manage (new Gtk::Image);
  kas_images[1]  = Gtk::manage (new Gtk::Image);
  score_label[1] = Gtk::manage (new Gtk::Label);
  score_label[0] = Gtk::manage (new Gtk::Label);
  round_label    = Gtk::manage (new Gtk::Label);
  round_label->set_justify (Gtk::JUSTIFY_CENTER);

  side_panel->pack_start (*(score_label[0]), Gtk::PACK_SHRINK, 5);
  side_panel->pack_start (*(Gtk::manage (new Gtk::Label)), Gtk::PACK_EXPAND_PADDING, 0);
  side_panel->pack_start (*(kas_images[0]), Gtk::PACK_SHRINK, 0);
  side_panel->pack_start (*round_label, Gtk::PACK_SHRINK, 0);  
  side_panel->pack_start (*(kas_images[1]), Gtk::PACK_SHRINK, 0);
  side_panel->pack_start (*(Gtk::manage (new Gtk::Label)), Gtk::PACK_EXPAND_PADDING, 0);
  side_panel->pack_start (*(score_label[1]), Gtk::PACK_SHRINK, 5);

  set_resizable(false);
  show_all_children();
}

Window::~Window()
{
  if (about_dialog) delete about_dialog;
  if (demo_dialog) delete demo_dialog;
  delete recording;
  delete brain_setter;
  delete brain;
}

void
Window::on_realize () 
{
  Gtk::Window::on_realize ();
  
  // Internal signals
  pattern_selector->pattern_changed_signal.connect
    (sigc::mem_fun (*this, &Window::on_pattern_changed));
  brain->got_move_signal.connect
    (sigc::mem_fun (*this, &Window::on_got_move));
  board_gui->next_move_signal.connect
    (sigc::mem_fun (*this, &Window::on_next_move));
  num_scheduled_moves = 0;

  // Defaults
  pattern_selector->on_selection ();
  on_action (ACTION_INIT);
  display_scores ();

  // Periodic event
  sigc::slot<bool> timeout_slot = 
    sigc::bind (sigc::mem_fun (*this, &Window::on_timeout), (int) TIMEOUT_ANIMATION);
  Glib::signal_timeout().connect (timeout_slot, 1000 / TICKS_PER_SEC);  // 30 times per sec 
}

void
Window::display_scores ()
{
  for (int i=0; i < 2; i++) {
    score_label[i]->set_markup (String::ucompose ("<big><b>%1</b></big>", num_win[i]));
    kas_images[i]->set (board_gui->theme.kas_image[i ^ board_gui->rotated]);
  }
  Board& board = board_gui->board;
  GameStageType stage = board.get_stage();
  Glib::ustring round_text = String::ucompose (
    "<big><b>%1</b></big>", 
    String::ucompose (_("Round\n%1"), stage == STAGE_GAME_OVER ? num_rounds : num_rounds + 1));
  if (stage != STAGE_OPENING || board.get_player () == 1) {  
    round_label->set_markup (String::ucompose (
      "%1\n\n%2\n\n%3",
      board_gui->get_score (PLAYER_SIDE_TOP),
      round_text,
      board_gui->get_score (PLAYER_SIDE_BOTTOM)));
  }
  else {
    round_label->set_markup (round_text);
  }
}

void 
Window::stop_thinking ()
{
  brain->request_stop ();  // Abort any current thinking
  request_id++;            // Ensure result from aborted thinking is dismissed
}

void 
Window::on_got_move ()
{
  if (request_id == brain->response_id) {   // Dismiss move from aborted thinking
    const Move *move = brain->response;
    board_gui->simulate (*brain->response);
  }
}

void 
Window::on_next_move (std::string* move_notation)
{
  Board& board = board_gui->board;
  GameStageType stage = board.get_stage();
  // Record the move (completed move only)
  if (move_notation && stage != STAGE_RESOLVE && stage != STAGE_RESOLVE_KAS_SELECTION) {
    recording->record (*move_notation);
  }
  // Update data
  if (stage == STAGE_GAME_OVER) {
    num_rounds++;
    // Who won?
    int diff = board_gui->get_score (PLAYER_SIDE_TOP) - board_gui->get_score (PLAYER_SIDE_BOTTOM);
    if (diff > 0)      num_win[0]++;
    else if (diff < 0) num_win[1]++;
  }
  if (stage != STAGE_OPENING || board.get_player () == 1) {
    action_group->get_action("Pass")->set_sensitive (false);
    pattern_selector->set_sensitive (false);
    display_scores ();
  }
  // Ready for next round
  if (stage == STAGE_GAME_OVER) {
    pattern_selector->set_sensitive (mode == USER_VS_AI);
    pattern_selector->unselect ();
    if (mode == AI_VS_AI) {
      // 5 seconds wait before initializing the next round
      sigc::slot<bool> timeout_slot = 
        sigc::bind (sigc::mem_fun (*this, &Window::on_timeout), (int) TIMEOUT_GAME_OVER);
      Glib::signal_timeout().connect (timeout_slot, 5000);
      num_scheduled_moves++;
    }
    return;
  }
  if (board_gui->get_player_type () == PLAYER_VIRTUAL) {
    request_id++;
    if (mode == AI_VS_AI) {
      // board_gui->theme.random (); // jaidi : uncomment this for stress test
      brain->request_think (board, request_id, demo_player[board_gui->get_player_side ()]);
    }
    else
      brain->request_think (board, request_id, virtual_player);
  }
}

void 
Window::on_pattern_changed (const Matrix m)
{
  char *pattern_name = _(pattern_selector->pattern_name ());
  if (pattern_selector->is_randomly_generated ()) {
    statusbar->set_markup (String::ucompose (_("Random Pattern: %1"), pattern_name));
  }
  else {
    statusbar->set_markup (String::ucompose (_("Pattern: \"%1\""), pattern_name));
  }
  if (mode==USER_VS_AI) {
    // Make a restore point
    undo_num_rounds = num_rounds;
    for (int i=0; i < 2; i++) {
      undo_num_win[i] = num_win[i];
    }
    board_gui->set_turn (PLAYER_VIRTUAL, PLAYER_HUMAN, 1);
    action_group->get_action("Pass")->set_sensitive (true);
  }
  stop_thinking ();
  board_gui->on_pattern_changed (m);
  recording->record (board_gui->board);
  if (mode == AI_VS_AI) {
    // 5 seconds wait before calling on_next_move
    sigc::slot<bool> timeout_slot = 
      sigc::bind (sigc::mem_fun (*this, &Window::on_timeout), (int) TIMEOUT_GAME_START);
    Glib::signal_timeout().connect (timeout_slot, 5000);
    num_scheduled_moves++;
  }
  else {
    on_next_move (0);
  }
  display_scores ();
}

bool 
Window::on_timeout (int timerNumber)
{
  // [Undo] and [Next Round] button
  action_group->get_action("Undo")->set_sensitive (
    mode == USER_VS_AI &&
      (board_gui->board.get_player () != 0 || board_gui->board.get_stage () != STAGE_OPENING));
  action_group->get_action("NextRound")->set_sensitive (
    mode == USER_VS_AI && board_gui->board.get_stage () == STAGE_GAME_OVER);

  if (timerNumber == TIMEOUT_ANIMATION) {
    if (board_gui->theme.change ()) {
      board_gui->set_size_request (board_gui->theme.board_image_width (), 
                                   board_gui->theme.board_image_width ());
      board_gui->queue_draw ();
      display_scores ();
    }
    else {
      board_gui->animate ();
    }
    pattern_selector->periodic ();
    return true;   // periodic
  }
  
  if (timerNumber == TIMEOUT_GAME_OVER) {
    // previously: g_assert(mode != AI_VS_AI)
    // The assertion fails when a demo game is just finishing when
    // the player issues the New Game command.
    num_scheduled_moves--;
    if (num_scheduled_moves > 0 || mode != AI_VS_AI) return false;
    board_gui->set_turn (PLAYER_VIRTUAL, PLAYER_VIRTUAL, num_rounds&1);  // Alternate first player
    board_gui->theme.random ();
    board_gui->queue_draw ();
    pattern_selector->random ();
    return false;  // kill timer
  }
  
  if (timerNumber == TIMEOUT_GAME_START) {
    num_scheduled_moves--;
    if (num_scheduled_moves > 0) return false;
    on_next_move (0);
    return false;
  }
  g_assert_not_reached();
  return false;
}

void 
Window::create_menus ()
{
  Gtk::StockID null_stock;
  struct Action {
    const char   *key;
    const char   *label;
    Gtk::StockID  stock;
    ActionID      id;
    const char   *accel;
  }
  action_list[] = {
    {"<Game",       N_("_Game") },
      {"NewGame",   N_("_New Game..."),          Gtk::Stock::NEW,    ACTION_NEW_GAME,  NULL },
      {"Demo",      N_("_Demo..."),              null_stock,         ACTION_DEMO,      NULL },
      {"-"},
      {"QuitGame",  N_("_Quit"),                 Gtk::Stock::QUIT,   ACTION_QUIT,      NULL },
    {">"},
    {"<Move",     N_("_Move") },
      {"NextRound", N_("_Next Round"),           null_stock,         ACTION_NEXT_ROUND, NULL },
      {"Pass",      N_("Computer _Move First"),  null_stock,         ACTION_PASS,      NULL },
      {"Undo",      N_("_Undo Move"),            Gtk::Stock::UNDO,   ACTION_UNDO,      "<control>Z" },
    {">"},
    //{"<Edit",       N_("_Edit") },
    // {"Copy",      N_("_Copy"),                  Gtk::Stock::COPY,   ACTION_COPY },
    //{">"},
    
    
    {
    "<Preferences",N_("Preferences") },
      {"Theme",     N_("Theme..."),              null_stock,         ACTION_THEME,     NULL },
      {"<Zoom",     N_("Board Size") },
        {"Zoom50",  N_("50%"),                   null_stock,         ACTION_ZOOM50,    NULL },
        {"Zoom75",  N_("75%"),                   null_stock,         ACTION_ZOOM75,    NULL },
        {"Zoom100", N_("100%"),                  null_stock,         ACTION_ZOOM100,   NULL },
      {">"},
    {">"},
    {"<Help",       N_("_Help") },
      {"HowToPlay", N_("_Contents"),             Gtk::Stock::HELP,   ACTION_HOW_TO_PLAY, "F1" },
      {"About",     N_("_About"),                Gtk::Stock::ABOUT,  ACTION_ABOUT,     NULL },
    {">"},
    {NULL}
  };
  
  action_group = Gtk::ActionGroup::create ();
  std::string menu_xml = "";
  
  for (Action *action=action_list; action->key; action++) {
    const char *key = action->key;
    if (key[0] == '-') {
      menu_xml += "<separator />";
    }
    else if (key[0] == '>') {
      menu_xml += "</menu>";
    }
    else if (key[0] == '<') {
      action_group->add (Gtk::Action::create (key+1, _(action->label)));
      menu_xml += String::ucompose ("<menu action='%1'>", key+1);
    }
    else {
      Glib::RefPtr<Gtk::Action> gtk_action = (action->stock == null_stock) 
          ? Gtk::Action::create (key, _(action->label))
          : Gtk::Action::create (key, action->stock, _(action->label));
      if (action->accel) {
        action_group->add (gtk_action, Gtk::AccelKey(action->accel),
          sigc::bind (sigc::mem_fun (*this, &Window::on_action), (int) action->id));
      }
      else {
        action_group->add (gtk_action, 
          sigc::bind (sigc::mem_fun (*this, &Window::on_action), (int) action->id));
      }
      menu_xml += String::ucompose ("<menuitem action='%1' />", key);
    }
  }

  ui_manager = Gtk::UIManager::create ();
  ui_manager->insert_action_group (action_group);
	add_accel_group (ui_manager->get_accel_group ());
  try
  {
    ui_manager->add_ui_from_string (String::ucompose (
        "<ui>"
        "  <menubar name='MenuBar'>"
        "    %1 "
        "  </menubar>"
        "  <toolbar name='Toolbar'>"
        "    <toolitem action='NextRound' />"
        "    <toolitem action='Pass' />"
        "  </toolbar>"
        "</ui>", menu_xml)
    );      
  }
  catch (const Glib::Error& ex)
  {
    printf ("Fail to build menu\n");
    g_assert_not_reached ();
  }
}

void 
Window::on_action (int n)
{
  switch (n) {
    case ACTION_INIT :
    
    case ACTION_NEW_GAME : 
      if (n == ACTION_NEW_GAME && 
         !brain_setter->show_dialog (_("Choose Opponent"), *this, virtual_player))
        break;
      else
        mode = USER_VS_AI;
      // Reset scores
      num_rounds = 0;
      for (int i=0; i < 2; i++) {
        num_win[i] = 0;
      }
      // no break
      
    case ACTION_NEXT_ROUND :
      stop_thinking ();
      g_return_if_fail (mode == USER_VS_AI);
      pattern_selector->set_sensitive (true);
      pattern_selector->on_selection ();
      break;
      
    case ACTION_DEMO :
      if (!show_demo_dialog ()) break;
      // Reset scores
      num_rounds = 0;
      for (int i=0; i < 2; i++) {
        num_win[i] = 0;
      }
      stop_thinking ();
      board_gui->stop_simulation ();
      mode = AI_VS_AI;
      action_group->get_action("Pass")->set_sensitive (false);
      pattern_selector->set_sensitive (false);
      board_gui->set_turn (PLAYER_VIRTUAL, PLAYER_VIRTUAL, 1);
      pattern_selector->random ();
      break;
      
    case ACTION_PASS :
      stop_thinking ();
      g_return_if_fail (mode == USER_VS_AI);
      action_group->get_action("Pass")->set_sensitive (false);
      pattern_selector->set_sensitive (false);
      board_gui->set_turn (PLAYER_VIRTUAL, PLAYER_HUMAN, 0);
      on_next_move (0);
      break;
      
    case ACTION_UNDO :
      stop_thinking ();
      g_return_if_fail (mode == USER_VS_AI);
      board_gui->stop_simulation ();
      // Apply restore point
      num_rounds = undo_num_rounds;
      for (int i=0; i < 2; i++) {
        num_win[i] = undo_num_win[i];
      }
      recording->undo (board_gui->board);
      if (board_gui->get_player_type () == PLAYER_VIRTUAL) {
        recording->undo (board_gui->board);
      }
      if (board_gui->board.get_player () == 0 && board_gui->board.get_stage () == STAGE_OPENING) {
        // Undo to game start
        board_gui->set_turn (PLAYER_VIRTUAL, PLAYER_HUMAN, 1);
        action_group->get_action("Pass")->set_sensitive (true);
        pattern_selector->set_sensitive (true);
        pattern_selector->on_selection ();
      }
      else {
        on_next_move (0);
      }
      break;
      
    case ACTION_THEME :
      board_gui->theme.show_dialog (*this);
      break;
      
    case ACTION_ZOOM50 :
      board_gui->theme.zoom (50);
      break;
      
    case ACTION_ZOOM75 :
      board_gui->theme.zoom (75);
      break;

    case ACTION_ZOOM100 :
      board_gui->theme.zoom (100);
      break;

    case ACTION_QUIT :
      stop_thinking ();
      hide ();
      break;
      
    case ACTION_COPY :
      copy ();
      break;
      
    case ACTION_HOW_TO_PLAY :
      {
        GError *error = NULL;
        if (!gnome_help_display ("pasang-emas.xml", "", &error)) {
          printf("Error: %s \n", error->message);
          g_error_free (error);
        }
        
        //jaidi: this is for future when libgnomeui is finally killed  
        //screen = gtk_widget_get_screen (get pasang-emas window);
        //gtk_show_uri (screen, "ghelp:pasang-emas",  gtk_get_current_event_time (), &error);
        //if (error != NULL) {
        //  printf("Error: %s \n", error->message);
        //  g_error_free (error);
        //}
      }
      break;
      
    case ACTION_ABOUT :
      show_about_dialog ();
      break;
      
    default :
      printf ("action %d not implemented\n", n);
      g_assert_not_reached ();
  }
  display_scores ();
}


void
Window::copy ()
{ 
  std::string xml = 
    "<?xml version=\"1.0\"?>\n"
    "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"\n"
    " \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n"
    "<svg width=\"12cm\" height=\"12cm\" viewBox=\"0 0 1200 1200\" version=\"1.1\"\n"
    "     xmlns=\"http://www.w3.org/2000/svg\">\n"
    "  <rect x=\"0\" y=\"0\" width=\"1200\" height=\"1200\"\n"
    "        fill=\"white\" stroke=\"black\" stroke-width=\"5\" />\n"
    "  <g style=\"stroke:black; stroke-width:5\">\n";
  char px[10];
  char py[10];
  for (int i=1; i <= 11; i++) {
    sprintf (px, "%d", i * 100);
    xml += String::ucompose (i == 6 ?
        "    <line x1=\"%1\" y1=\"100\" x2=\"%1\" y2=\"500\" />\n"
        "    <line x1=\"%1\" y1=\"700\" x2=\"%1\" y2=\"1100\" />\n"
        "    <line y1=\"%1\" x1=\"100\" y2=\"%1\" x2=\"500\" />\n"
        "    <line y1=\"%1\" x1=\"700\" y2=\"%1\" x2=\"1100\" />\n"
        :
        "    <line x1=\"%1\" y1=\"100\" x2=\"%1\" y2=\"1100\" />\n"
        "    <line y1=\"%1\" x1=\"100\" y2=\"%1\" x2=\"1100\" />\n",
      px);
  }
  xml += 
        "  </g>\n"
        "  <g style=\"stroke:black; stroke-width:10\">\n";
  for (int i=1; i <= 11; i++) {
    sprintf (px, "%d", (12 - i) * 100);
    for (int j=1; j <= 11; j++) {
      sprintf (py, "%d", (12 - j) * 100);
      PieceType piece = board_gui->board.get_piece (j, i);
      xml += String::ucompose (
        piece == PIECE_BLACK ? "    <circle cx=\"%1\" cy=\"%2\" r=\"35\" fill=\"black\" />\n" :
        piece == PIECE_WHITE ? "    <circle cx=\"%1\" cy=\"%2\" r=\"35\" fill=\"white\" />\n" :
        piece == PIECE_KAS_0 ? "    <circle cx=\"%1\" cy=\"%2\" r=\"35\" fill=\"red\" />\n" :
        piece == PIECE_KAS_1 ? "    <circle cx=\"%1\" cy=\"%2\" r=\"35\" fill=\"blue\" />\n" :
        /* empty */            "",
        px, py);
    }
  }
  xml += 
        "  </g>\n"
        "</svg>";
  
  Glib::RefPtr<Gtk::Clipboard> clipboard = Gtk::Clipboard::get ();
  clipboard->set_text (xml);
}


bool 
Window::show_demo_dialog ()
{
  if (demo_dialog == NULL) {
    demo_dialog = new Gtk::Dialog (_("Choose Players"), *this, true, true);
    demo_dialog->set_resizable (false);
    Gtk::VBox* vbox = demo_dialog->get_vbox ();

    // Set up dialog content
    Gtk::Table* table = Gtk::manage (new Gtk::Table (5, 2, false));
    table->set_row_spacings (10);
    table->set_col_spacings (5);
    vbox->pack_start (*table, Gtk::EXPAND, 5);
    for (int i=0; i < 2; i++) {
      demo_player_labels[i] = Gtk::manage (new Gtk::Label);
      table->attach (*(demo_player_labels[i]), 0, 1, i*2+1, i*2+2);
      Gtk::Button* button = Gtk::manage (new Gtk::Button (_("Change...")));
      table->attach (*button, 1, 2, i*2+1, i*2+2);
      button->signal_clicked().connect ( 
        sigc::bind (sigc::mem_fun (*this, &Window::show_demo_subdialog), i));
    }
    table->attach (*Gtk::manage (new Gtk::Label (_("vs"))), 0, 1, 2, 3);
    
    demo_dialog->add_button (Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    demo_dialog->add_button (Gtk::Stock::OK, Gtk::RESPONSE_OK);
    demo_dialog->show_all_children ();
  }

  VirtualPlayer demo_player_save[2];
  for (int i=0; i < 2; i++) {
    demo_player_save[i] = demo_player[i];
    demo_player_labels[i]->set_text (brain_setter->get_name (demo_player[i]));
  }
  int response = demo_dialog->run ();
  demo_dialog->hide ();
  if (response == Gtk::RESPONSE_OK) {
    return true;
  }
  else {
    for (int i=0; i < 2; i++) demo_player[i] = demo_player_save[i];
    return false;
  }
}

void 
Window::show_demo_subdialog (int p)
{
  g_assert (p==0 || p==1);
  if (brain_setter->show_dialog (_("Choose Player"), *demo_dialog, demo_player[p])) {
    demo_player_labels[p]->set_text (brain_setter->get_name (demo_player[p]));
  }
}

void 
Window::show_about_dialog ()
{
  if (about_dialog == NULL) {
    about_dialog = new Gtk::AboutDialog;
  	about_dialog->set_transient_for (*this);
  	about_dialog->set_modal (false);
  	about_dialog->set_name (_(PACKAGE_NAME));
  	about_dialog->set_version (PACKAGE_VERSION);
  	about_dialog->set_comments (_("A traditional board game of Brunei"));
  	about_dialog->set_copyright ("Copyright (C) 2008 Nor Jaidi Tuah <norjaidi.tuah@ubd.edu.bn>");
  	try {
    	about_dialog->set_logo(Gdk::Pixbuf::create_from_file (PIXMAPSDIR"/pasang-emas-flash.png"));
    }
    catch (Glib::Error e) {
      printf("File error: %s\n", e.what().c_str());
    }

  	std::deque<Glib::ustring> authors;
  	authors.push_back ("Programmer:");
  	authors.push_back ("  Nor Jaidi Tuah <norjaidi.tuah@ubd.edu.bn>");
  	authors.push_back (" ");
  	authors.push_back ("Initial idea:");
  	authors.push_back ("  Mohd Abdoh bin Haji Awang Damit");
  	
  	std::deque<Glib::ustring> artists;
  	artists.push_back ("  Nor Jaidi Tuah <norjaidi.tuah@ubd.edu.bn>");

  	about_dialog->set_authors (authors);
  	about_dialog->set_artists (artists);

    Glib::ustring translator_credits = _("translator_credits");
    if (translator_credits != "translator_credits") {
      about_dialog->set_translator_credits (translator_credits);
    }

  	about_dialog->set_license (
  		"This program is free software; you can redistribute it\n"
  		"and/or modify it under the terms of the GNU General Public\n"
  		"License as published by the Free Software Foundation; either\n"
  		"version 3 of the License, or (at your option) any later\n"
  		"version.\n"
  		"\n"
  		"This program is distributed in the hope that it will be\n"
  		"useful, but WITHOUT ANY WARRANTY; without even the implied\n"
  		"warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR\n"
  		"PURPOSE.  See the GNU General Public License for more details."
  	);
    about_dialog->signal_response().connect(sigc::mem_fun(*this, &Window::on_about_dialog_closed));
  }
	about_dialog->show ();
}

void 
Window::on_about_dialog_closed (int response) 
{
  about_dialog->hide();
}

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
